//
//  DemoShareSoundNetworkManager.m
//  OpenSDKDemo
//
//  Created by Stephen Boyle on 4/23/21.
//  Copyright © 2021 bytedance.co. All rights reserved.
//

#import "DemoShareSoundNetworkManager.h"
#import "DemoNetworkManager.h"
#import "UIAlertController+Addition.h"
#import <TikTokOpenSDK/TikTokOpenSDKAuth.h>

#define kTikTokOpenSDKFileUploaderBoundary @"ttopensdk"
#define kTikTokOpenSDKFileUploaderNewline [[NSString stringWithFormat:@"\r\n"] dataUsingEncoding:NSUTF8StringEncoding]

@implementation TikTokShareSoundResponse

+ (NSDictionary *)JSONKeyPathsByPropertyKey {
    return @{
        @"statusCode": @"status_code",
        @"statusMsg": @"status_message",
        @"data": @"data"
    };
}

@end

@implementation DemoShareSoundNetworkManager

+ (void)startShareSoundProcessOnVC:(UIViewController *)vc
{
    // 1. auth
    TikTokOpenSDKAuthRequest *request = [[TikTokOpenSDKAuthRequest alloc] init];
    NSArray *scopes = @[@"user.info.basic",@"share.sound.create"];
    NSOrderedSet *scopesSet = [NSOrderedSet orderedSetWithArray:scopes];
    request.permissions = scopesSet;
    [request sendAuthRequestViewController:vc completion:^(TikTokOpenSDKAuthResponse * _Nonnull resp) {
        if (resp.errCode != 0) {
            NSString *alertString = [NSString stringWithFormat:@"Author failed code : %@, msg : %@", @(resp.errCode), resp.errString];
            [self p_showMsg:alertString inVC:vc];
        } else {
            [DemoNetworkManager authLogin:resp.code
                               completion:^(NSString * _Nonnull accessToken, NSString * _Nonnull openID) {
                if (!accessToken || accessToken.length == 0) {
                    [self p_showMsg:@"Fail to get the auth token!" inVC:vc];
                }
                // 2. read music file
                NSString *musicFilePath = [[NSBundle mainBundle] pathForResource:@"demo" ofType:@"mp3"];
                // 3.1 build URL
                NSString *uploadURLStr = [NSString stringWithFormat:@"%@%@?access_token=%@&open_id=%@", [self p_networkDomain], [self p_uploadMusicPath], accessToken, openID];
                NSURL *url = [NSURL URLWithString:uploadURLStr];

                // 3.2 build request
                NSMutableURLRequest *request = [NSMutableURLRequest requestWithURL:url];
                request.HTTPMethod = @"POST";
                [request setValue:[NSString stringWithFormat:@"multipart/form-data;boundary=%@",kTikTokOpenSDKFileUploaderBoundary] forHTTPHeaderField:@"Content-Type"];
                NSMutableData *data = [[NSMutableData alloc] init];

                // 3.3 file data
                [data appendData:[[NSString stringWithFormat:@"--%@", kTikTokOpenSDKFileUploaderBoundary] dataUsingEncoding:NSUTF8StringEncoding]];
                [data appendData:kTikTokOpenSDKFileUploaderNewline];
                // get file name
                NSArray<NSString *> *filePathComponents = [musicFilePath componentsSeparatedByString:@"/"];
                NSString *fileName = musicFilePath;
                if (filePathComponents.count > 0) {
                    fileName = filePathComponents[filePathComponents.count - 1];
                }
                // setup the file data section
                [data appendData:[[NSString stringWithFormat:@"Content-Disposition: form-data; name=\"sound_file\"; filename=\"%@\"",fileName] dataUsingEncoding:NSUTF8StringEncoding]];
                [data appendData:kTikTokOpenSDKFileUploaderNewline];
                [data appendData:[@"Content-Type: application/octet-stream" dataUsingEncoding:NSUTF8StringEncoding]];
                [data appendData:kTikTokOpenSDKFileUploaderNewline];
                [data appendData:[@"Content-Transfer-Encoding:binary" dataUsingEncoding:NSUTF8StringEncoding]];
                [data appendData:kTikTokOpenSDKFileUploaderNewline];
                [data appendData:kTikTokOpenSDKFileUploaderNewline];
                [data appendData:[NSData dataWithContentsOfFile:musicFilePath]];
                [data appendData:kTikTokOpenSDKFileUploaderNewline];
                // end of the content
                [data appendData:[[NSString stringWithFormat:@"--%@--",kTikTokOpenSDKFileUploaderBoundary] dataUsingEncoding:NSUTF8StringEncoding]];
                request.HTTPBody = data;
                NSURLSession *fileUploadSession = [NSURLSession sharedSession];
                NSURLSessionDataTask *task = [fileUploadSession dataTaskWithRequest:request completionHandler:^(NSData * _Nullable data,
                                                                                                                NSURLResponse * _Nullable response,
                                                                                                                NSError * _Nullable error) {
                    NSString *alertString = nil;
                    if (!error) {
                        NSError *jsonError = nil;
                        NSData *dataUTF8 = [[[NSString alloc] initWithData:data encoding:NSISOLatin1StringEncoding] dataUsingEncoding:NSUTF8StringEncoding];
                        NSDictionary *jsonDict = [NSJSONSerialization JSONObjectWithData:dataUTF8 options:0 error:&jsonError];
                        TikTokShareSoundResponse *resp = [MTLJSONAdapter modelOfClass:TikTokShareSoundResponse.class fromJSONDictionary:jsonDict error:&jsonError];
                        if (jsonError || !resp) {
                            alertString = @"Network request error while uploading!";
                        } else {
                            alertString = @"Share sound succeed!";
                        }
                    } else {
                        alertString = [NSString stringWithFormat:@"Share sound error: code: %@, msg : %@", @(resp.errCode), resp.errString];
                    }
                    [self p_showMsg:alertString inVC:vc];
                }];
                [task resume];
            }];
        }
    }];
}

#pragma mark - Private

+ (void)p_showMsg:(NSString *)msg inVC:(UIViewController *)vc
{
    dispatch_async(dispatch_get_main_queue(), ^{
        [UIAlertController showMsg:msg inVC:vc];
    });
}

+ (NSString *)p_networkDomain
{
    return @"https://open-api.tiktok.com";
}

+ (NSString *)p_uploadMusicPath
{
    return @"/share/sound/upload";
}

@end
