//
//  OpenPlatformShareViewController.m
//  OpenSDKDemo_Example
//
//  Created by Jiny.du on 2019/7/22.
//  Copyright © 2019 bytedance.co All rights reserved.
//
#import <TikTokOpenSDK/TikTokOpenSDKAuth.h>
#import "OpenPlatformActionView.h"
#import "OpenPlatformCellProtocol.h"
#import "OpenPlatformSectionCell.h"
#import "OpenPlatformSession.h"
#import "OpenPlatformShareDataControl.h"
#import "OpenPlatformShareViewController.h"
#import "OpenPlatformUIDefine.h"
#import "QBImagePickerController.h"
#import "UIAlertController+Addition.h"
#import "DemoNetworkManager.h"
#import <Masonry/Masonry.h>

@interface OpenPlatformShareViewController () <QBImagePickerControllerDelegate, UITableViewDelegate, UITableViewDataSource>
@property (nonatomic, strong) UITableView *shareInfoTableView;
@property (nonatomic, strong) OpenPlatformShareDataControl *shareDataControl;
@property (nonatomic, strong) OpenPlatformActionView *actionView;

@end

@implementation OpenPlatformShareViewController

- (void)viewDidLoad
{
    [super viewDidLoad];
    [self p_setupUI];
    [self p_setupConstraints];
}

#pragma mark - Private

- (void)p_setupUI
{
    self.view.backgroundColor = [UIColor demo_colorWithHex:@"#f5f5f5"];
    self.title = @"Share";
    [self.view addSubview:self.shareInfoTableView];
    [self.view addSubview:self.actionView];
    [self.shareInfoTableView reloadData];
}

- (void)p_setupConstraints
{
    [self.shareInfoTableView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.top.left.right.equalTo(self.view);
        make.bottom.equalTo(self.actionView.mas_top);
    }];
    [self.actionView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(self.view).offset(10);
        make.right.equalTo(self.view).offset(-10);
        make.bottom.equalTo(self.view).offset(-(BOTTOM_SAFEAREA_HEIGHT + 20));
        make.height.equalTo(@(50));
    }];
}

#pragma mark - Actions

- (void)tapShareImage
{
    [self showImageSelectVCWithMediaType:QBImagePickerMediaTypeImage];
}

- (void)tapShareVideo
{
    [self showImageSelectVCWithMediaType:QBImagePickerMediaTypeVideo];
}

- (void)showImageSelectVCWithMediaType:(QBImagePickerMediaType)mediaType
{
    QBImagePickerController *imagePickerVC = [[QBImagePickerController alloc] init];
    imagePickerVC.delegate = self;
    imagePickerVC.allowsMultipleSelection = YES;
    imagePickerVC.maximumNumberOfSelection = 35;
    imagePickerVC.showsNumberOfSelectedAssets = YES;
    imagePickerVC.mediaType = mediaType;
    [self presentViewController:imagePickerVC animated:YES completion:NULL];
}

#pragma mark - TableView Delegate

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath
{
    [tableView deselectRowAtIndexPath:indexPath animated:NO];
}

- (CGFloat)tableView:(UITableView *)tableView heightForRowAtIndexPath:(NSIndexPath *)indexPath
{
    return self.shareDataControl.sessions[indexPath.section].heightForSession;
}

- (CGFloat)tableView:(UITableView *)tableView heightForFooterInSection:(NSInteger)section
{
    return .1f;
}

- (CGFloat)tableView:(UITableView *)tableView heightForHeaderInSection:(NSInteger)section
{
    return 25.f;
}

- (UIView *)tableView:(UITableView *)tableView viewForFooterInSection:(NSInteger)section
{
    return [[UIView alloc] init];
}

- (void)scrollViewDidScroll:(UIScrollView *)scrollView
{
    [self.view endEditing:YES];
}

#pragma mark - TableView DataSource

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
    OpenPlatformSession *item = self.shareDataControl.sessions[indexPath.row];
    UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:NSStringFromClass([OpenPlatformSession class])];
    if ([cell conformsToProtocol:@protocol(OpenPlatformSectionProtocol)]) {
        [((id<OpenPlatformSectionProtocol>)cell) configWithSectionItem:item];
    }

    return cell;
}

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView
{
    return 1;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
    return self.shareDataControl.sessions.count;
}

#pragma mark - QBImagePickerControllerDelegate

- (void)qb_imagePickerController:(QBImagePickerController *)imagePickerController didFinishPickingAssets:(NSArray *)assets
{
    NSMutableArray *ids = [NSMutableArray array];
    for (PHAsset *asset in assets) {
        if (![asset isKindOfClass:[PHAsset class]]) {
            NSAssert(NO, @"");
            continue;
        }

        [ids addObject:asset.localIdentifier];
    }

    if (imagePickerController.mediaType == QBImagePickerMediaTypeVideo) {
        [self.shareDataControl shareMediaIds:ids type:TikTokOpenSDKShareMediaTypeVideo];
    } else if (imagePickerController.mediaType == QBImagePickerMediaTypeImage) {
        [self.shareDataControl shareMediaIds:ids type:TikTokOpenSDKShareMediaTypeImage];
    }

    [imagePickerController dismissViewControllerAnimated:YES completion:nil];
}

- (void)qb_imagePickerControllerDidCancel:(QBImagePickerController *)imagePickerController
{
    [imagePickerController dismissViewControllerAnimated:YES completion:nil];
}

#pragma mark - Getters

- (UITableView *)shareInfoTableView
{
    if (!_shareInfoTableView) {
        _shareInfoTableView = [[UITableView alloc] initWithFrame:self.view.frame
                                                           style:UITableViewStyleGrouped];
        [_shareInfoTableView registerClass:[OpenPlatformSectionCell class]
                    forCellReuseIdentifier:@"OpenPlatformSession"];
        _shareInfoTableView.dataSource = self;
        _shareInfoTableView.delegate = self;
        _shareInfoTableView.backgroundColor = YAMBGDefualtColor;
        _shareInfoTableView.separatorStyle = UITableViewCellSeparatorStyleNone;
        _shareInfoTableView.contentInset = UIEdgeInsetsMake(0, 0, 20, 0);
    }

    return _shareInfoTableView;
}

- (OpenPlatformShareDataControl *)shareDataControl
{
    if (!_shareDataControl) {
        _shareDataControl = [[OpenPlatformShareDataControl alloc] init];
        __weak typeof(self) weakSelf = self;
        _shareDataControl.dataSourceChange = ^(NSIndexPath *_Nonnull indexPath) {
            __strong typeof (weakSelf) strongSelf = weakSelf;
            if (!strongSelf) {
                return;
            }

            if (!indexPath) {
                [strongSelf.shareInfoTableView reloadData];
                return;
            }

            UITableViewCell *cell = [strongSelf.shareInfoTableView cellForRowAtIndexPath:indexPath];
            if (cell) {
                [UIView performWithoutAnimation:^{
                    [strongSelf.shareInfoTableView reloadRowsAtIndexPaths:@[indexPath]
                                                         withRowAnimation:UITableViewRowAnimationNone];
                }];
            }
        };
    }

    return _shareDataControl;
}

- (OpenPlatformActionView *)actionView
{
    if (!_actionView) {
        _actionView = [[OpenPlatformActionView alloc] initWithNumberOfButtons:2];
        [[_actionView buttonAtIndex:0] setTitle:@"Share Image" forState:UIControlStateNormal];
        [[_actionView buttonAtIndex:0] addTarget:self action:@selector(tapShareImage) forControlEvents:(UIControlEventTouchUpInside)];
        [[_actionView buttonAtIndex:1] setTitle:@"Share Video" forState:UIControlStateNormal];
        [[_actionView buttonAtIndex:1] addTarget:self action:@selector(tapShareVideo) forControlEvents:UIControlEventTouchUpInside];
    }
    return _actionView;
}

@end
